"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.config = exports.updateConfig = exports.overrideDefaultConfig = void 0;
const fs = require("fs");
const Logger = require("./Logger");
const merge = require("deepmerge");
const minimist = require("minimist");
const path_1 = require("path");
const types_1 = require("@shardus/types");
let config = {
    ARCHIVER_IP: '127.0.0.1',
    ARCHIVER_PORT: 4000,
    ARCHIVER_HASH_KEY: '',
    ARCHIVER_PUBLIC_KEY: '',
    ARCHIVER_SECRET_KEY: '',
    ARCHIVER_LOGS: 'archiver-logs',
    ARCHIVER_DB: 'archiver-db',
    ARCHIVER_DATA: {
        cycleDB: 'cycles.sqlite3',
        accountDB: 'accounts.sqlite3',
        transactionDB: 'transactions.sqlite3',
        receiptDB: 'receipts.sqlite3',
        originalTxDataDB: 'originalTxsData.sqlite3',
        processedTxDB: 'processedTransactions.sqlite3',
        txDigestDB: 'txDigest.sqlite3',
    },
    DATASENDER_TIMEOUT: 1000 * 60 * 5,
    RATE_LIMIT: 100,
    N_NODE_REJECT_PERCENT: 5,
    N_NODELIST: 10,
    N_RANDOM_NODELIST_BUCKETS: 100,
    RECEIPT_CONFIRMATIONS: 5,
    STATISTICS: {
        save: true,
        interval: 1,
    },
    ARCHIVER_MODE: 'release',
    DevPublicKey: '',
    dataLogWrite: true,
    dataLogWriter: {
        dirName: 'data-logs',
        maxLogFiles: 10,
        maxReceiptEntries: 10000,
        maxCycleEntries: 500,
        maxOriginalTxEntries: 10000, // Should be >= max TPS experienced by the network.
    },
    experimentalSnapshot: true,
    VERBOSE: false,
    useSerialization: true,
    useSyncV2: true,
    sendActiveMessage: false,
    globalNetworkAccount: process.env.GLOBAL_ACCOUNT || '1000000000000000000000000000000000000000000000000000000000000001',
    maxValidatorsToServe: 10,
    limitToArchiversOnly: true,
    verifyReceiptData: true,
    verifyReceiptSignaturesSeparately: true,
    verifyAccountData: true,
    verifyAppReceiptData: true,
    skipGlobalTxReceiptVerification: true,
    REQUEST_LIMIT: {
        MAX_ACCOUNTS_PER_REQUEST: 1000,
        MAX_RECEIPTS_PER_REQUEST: 100,
        MAX_ORIGINAL_TXS_PER_REQUEST: 100,
        MAX_CYCLES_PER_REQUEST: 100,
        MAX_BETWEEN_CYCLES_PER_REQUEST: 100,
    },
    cycleRecordsCache: {
        enabled: false,
    },
    newPOQReceipt: false,
    storeReceiptBeforeStates: true,
    waitingTimeForMissingTxData: 2000,
    gossipToMoreArchivers: true,
    randomGossipArchiversCount: 2,
    subscribeToMoreConsensors: true,
    extraConsensorsToSubscribe: 1,
    saveOnlyGossipData: false,
    stopGossipTxData: false,
    usePOQo: true,
    requiredVotesPercentage: 2 / 3,
    requiredMajorityVotesPercentage: 60,
    maxCyclesShardDataToKeep: 10,
    configChangeMaxCyclesToKeep: 5,
    configChangeMaxChangesToKeep: 1000,
    receiptLoadTrakerInterval: 10 * 1000,
    receiptLoadTrakerLimit: 10,
    lastActivityCheckInterval: 15 * 1000,
    lastActivityCheckTimeout: 30 * 1000,
    txDigest: {
        cycleDiff: 10,
        syncDelay: 20,
        apiServerPort: 8084,
        txCronSchedule: '*/5 * * * *',
    },
    workerProcessesDebugLog: false,
    restrictFirstNodeSelectionByPublicKey: false,
    firstNodePublicKey: '',
    disableOffloadReceipt: false,
    disableOffloadReceiptForGlobalModification: true,
};
exports.config = config;
// Override default config params from config file, env vars, and cli args
async function overrideDefaultConfig(file) {
    const env = process.env;
    const args = process.argv;
    // Override config from config file
    try {
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        const fileConfig = types_1.Utils.safeJsonParse(fs.readFileSync(file, { encoding: 'utf8' }));
        const overwriteMerge = (target, source) => source;
        exports.config = config = merge(config, fileConfig, { arrayMerge: overwriteMerge });
    }
    catch (err) {
        if (err && err.code !== 'ENOENT') {
            console.warn('Failed to parse config file:', err);
        }
    }
    // Override config from env vars
    for (const param in config) {
        /* eslint-disable security/detect-object-injection */
        if (env[param]) {
            switch (typeof config[param]) {
                case 'number': {
                    config[param] = Number(env[param]);
                    break;
                }
                case 'string': {
                    config[param] = String(env[param]);
                    break;
                }
                case 'object': {
                    try {
                        const parameterStr = env[param];
                        if (parameterStr) {
                            const parameterObj = types_1.Utils.safeJsonParse(parameterStr);
                            config[param] = parameterObj;
                        }
                    }
                    catch (e) {
                        Logger.mainLogger.error(e);
                        Logger.mainLogger.error('Unable to JSON parse', env[param]);
                    }
                    break;
                }
                case 'boolean': {
                    config[param] = String(env[param]).toLowerCase() === 'true';
                    break;
                }
                default: {
                    break;
                }
            }
        }
    }
    // Override config from cli args
    const parsedArgs = minimist(args.slice(2));
    for (const param of Object.keys(config)) {
        /* eslint-disable security/detect-object-injection */
        if (parsedArgs[param]) {
            switch (typeof config[param]) {
                case 'number': {
                    config[param] = Number(parsedArgs[param]);
                    break;
                }
                case 'string': {
                    config[param] = String(parsedArgs[param]);
                    break;
                }
                case 'boolean': {
                    if (typeof parsedArgs[param] === 'boolean') {
                        config[param] = parsedArgs[param];
                    }
                    else {
                        config[param] = String(parsedArgs[param]).toLowerCase() === 'true';
                    }
                    break;
                }
                default: {
                    break;
                }
            }
        }
    }
    // Pull in secrets
    const secretsPath = (0, path_1.join)(__dirname, '../.secrets');
    const secrets = {};
    if (fs.existsSync(secretsPath)) {
        const lines = fs.readFileSync(secretsPath, 'utf-8').split('\n').filter(Boolean);
        lines.forEach((line) => {
            const [key, value] = line.split('=');
            secrets[key.trim()] = value.trim();
        });
        // Now, secrets contain your secrets, for example:
        // const apiKey = secrets.API_KEY;
        if (secrets['ARCHIVER_PUBLIC_KEY'])
            config.ARCHIVER_PUBLIC_KEY = secrets['ARCHIVER_PUBLIC_KEY'];
        if (secrets['ARCHIVER_SECRET_KEY'])
            config.ARCHIVER_SECRET_KEY = secrets['ARCHIVER_SECRET_KEY'];
        if (secrets['ARCHIVER_HASH_KEY'])
            config.ARCHIVER_HASH_KEY = secrets['ARCHIVER_HASH_KEY'];
    }
    if (config.ARCHIVER_HASH_KEY === '') {
        // Use default hash key if none provided
        // pragma: allowlist nextline secret
        config.ARCHIVER_HASH_KEY = '69fa4195670576c0160d660c3be36556ff8d504725be8a59b5a96509e0c994bc';
    }
    if (config.DevPublicKey === '') {
        // Use default dev public key if none provided
        // pragma: allowlist nextline secret
        config.DevPublicKey = '774491f80f47fedb119bb861601490f42bc3ea3b57fc63906c0d08e6d777a592';
    }
}
exports.overrideDefaultConfig = overrideDefaultConfig;
function updateConfig(newConfig) {
    for (const key in newConfig) {
        if (typeof newConfig[key] !== typeof config[key])
            throw new Error(`Value with incorrect type passed to update the Archiver Config: ${key}:${newConfig[key]} of type ${typeof newConfig[key]}`);
    }
    exports.config = config = merge(config, newConfig);
    Logger.mainLogger.info('Updated Archiver Config:', config);
    return config;
}
exports.updateConfig = updateConfig;
//# sourceMappingURL=Config.js.map