"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryCyleCount = exports.queryCycleRecordsBetween = exports.queryLatestCycleRecords = exports.queryCycleByMarker = exports.updateCycle = exports.bulkInsertCycles = exports.insertCycle = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
const serialization_1 = require("../utils/serialization");
async function insertCycle(cycle) {
    try {
        const fields = Object.keys(cycle).join(', ');
        const placeholders = Object.keys(cycle).fill('?').join(', ');
        const values = db.extractValues(cycle);
        const sql = 'INSERT OR REPLACE INTO cycles (' + fields + ') VALUES (' + placeholders + ')';
        await db.run(_1.cycleDatabase, sql, values);
        Logger.mainLogger.debug('Successfully inserted Cycle', cycle.cycleRecord.counter, cycle.cycleMarker);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert cycle or it is already stored in to database', cycle.cycleRecord.counter, cycle.cycleMarker);
    }
}
exports.insertCycle = insertCycle;
async function bulkInsertCycles(cycles) {
    try {
        const fields = Object.keys(cycles[0]).join(', ');
        const placeholders = Object.keys(cycles[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(cycles);
        let sql = 'INSERT OR REPLACE INTO cycles (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < cycles.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        await db.run(_1.cycleDatabase, sql, values);
        if (Config_1.config.VERBOSE)
            Logger.mainLogger.debug('Successfully inserted Cycles', cycles.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert Cycles', cycles.length);
    }
}
exports.bulkInsertCycles = bulkInsertCycles;
async function updateCycle(marker, cycle) {
    try {
        const sql = `UPDATE cycles SET counter = $counter, cycleRecord = $cycleRecord WHERE cycleMarker = $marker `;
        await db.run(_1.cycleDatabase, sql, {
            $counter: cycle.counter,
            $cycleRecord: cycle.cycleRecord && (0, serialization_1.SerializeToJsonString)(cycle.cycleRecord),
            $marker: marker,
        });
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Updated cycle for counter', cycle.cycleRecord.counter, cycle.cycleMarker);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to update Cycle', cycle.cycleMarker);
    }
}
exports.updateCycle = updateCycle;
async function queryCycleByMarker(marker) {
    try {
        const sql = `SELECT * FROM cycles WHERE cycleMarker=? LIMIT 1`;
        const dbCycle = (await db.get(_1.cycleDatabase, sql, [marker]));
        let cycle;
        if (dbCycle) {
            cycle = {
                counter: dbCycle.counter,
                cycleRecord: (0, serialization_1.DeSerializeFromJsonString)(dbCycle.cycleRecord),
                cycleMarker: dbCycle.cycleMarker,
            };
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('cycle marker', cycle);
        }
        return cycle;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryCycleByMarker = queryCycleByMarker;
async function queryLatestCycleRecords(count) {
    if (!Number.isInteger(count)) {
        Logger.mainLogger.error('queryLatestCycleRecords - Invalid count value');
        return [];
    }
    try {
        const sql = `SELECT * FROM cycles ORDER BY counter DESC LIMIT ${count ? count : 100}`;
        const dbCycles = (await db.all(_1.cycleDatabase, sql));
        const cycleRecords = [];
        if (dbCycles.length > 0) {
            for (const cycle of dbCycles) {
                if (cycle.cycleRecord)
                    cycleRecords.push((0, serialization_1.DeSerializeFromJsonString)(cycle.cycleRecord));
            }
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('cycle latest', cycleRecords);
        }
        return cycleRecords;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return [];
    }
}
exports.queryLatestCycleRecords = queryLatestCycleRecords;
async function queryCycleRecordsBetween(start, end) {
    try {
        const sql = `SELECT * FROM cycles WHERE counter BETWEEN ? AND ? ORDER BY counter ASC`;
        const dbCycles = (await db.all(_1.cycleDatabase, sql, [start, end]));
        const cycleRecords = [];
        if (dbCycles.length > 0) {
            for (const cycle of dbCycles) {
                if (cycle.cycleRecord)
                    cycleRecords.push((0, serialization_1.DeSerializeFromJsonString)(cycle.cycleRecord));
            }
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('cycle between', cycleRecords);
        }
        return cycleRecords;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return [];
    }
}
exports.queryCycleRecordsBetween = queryCycleRecordsBetween;
async function queryCyleCount() {
    let cycles;
    try {
        const sql = `SELECT COUNT(*) FROM cycles`;
        cycles = await db.get(_1.cycleDatabase, sql, []);
    }
    catch (e) {
        Logger.mainLogger.error(e);
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug('Cycle count', cycles);
    }
    if (cycles)
        cycles = cycles['COUNT(*)'];
    else
        cycles = 0;
    return cycles;
}
exports.queryCyleCount = queryCyleCount;
//# sourceMappingURL=cycles.js.map